// map.js
import {
  mapSvg,
  mapWidth,
  height,
  slider,
  display,
  nameMapping,
} from './globals.js';
import { updateLinePlot } from './plot.js';

let mapData;
let gdpDataMap;
let projection;
let graticule;
let path;
let mapColorAttr;
let mapColorScale;
let selectedCountry;

export function initMap(_mapData, _gdpDataMap) {
  mapData = _mapData;
  gdpDataMap = _gdpDataMap;

  projection = d3
    .geoEqualEarth()
    .fitSize([mapWidth, height], mapData);

  graticule = d3.geoGraticule();
  path = d3.geoPath(projection);

  let gdpValuesForAllYears = [];
  gdpDataMap.values().forEach((data) => {
    Object.keys(data).forEach((year) => {
      if (!isNaN(data[year])) {
        gdpValuesForAllYears.push(data[year]);
      }
    });
  });

  mapColorScale = d3
    .scaleLinear()
    .domain(d3.extent(gdpValuesForAllYears))
    .range(['white', 'blue']);

  // Draw the graticule
  mapSvg
    .append('path')
    .datum(graticule)
    .attr('class', 'graticule')
    .attr('d', d3.geoPath().projection(projection))
    .style('fill', 'none')
    .style('stroke', '#ccc')
    .style('stroke-width', 0.5);

  // Create group of paths
  mapSvg
    .append('g')
    .selectAll('path')
    .data(mapData.features)
    .join('path')
    .attr('id', (d) => d.properties.name)
    .attr('d', path)
    .attr('class', 'countries')
    .style('stroke', 'black')
    .style('stroke-width', 0.5)
    .on('mouseover', function (event, data) {
      d3.select(this).attr('fill', 'red');
      if (!gdpDataMap.has(data.properties.name)) {
        display.innerHTML =
          slider.value +
          ' ' +
          data.properties.name +
          ': No Data';
      } else {
        selectedCountry = data.properties.name;
        updateLinePlot();
        display.innerHTML =
          slider.value +
          ' ' +
          selectedCountry +
          ': ' +
          gdpDataMap.get(selectedCountry)[mapColorAttr];
      }
    })
    .on('mouseleave', function (event, data) {
      d3.select(this).attr(
        'fill',
        getCountryColor(data.properties.name),
      );
    });

  updateLinePlot();
}

function normalizeCountryName(name) {
  return nameMapping[name] || name;
}

function getCountryColor(name) {
  let entry = gdpDataMap.get(name);
  return entry == undefined
    ? 'grey'
    : mapColorScale(entry[mapColorAttr]);
}

export function renderMap() {
  mapColorAttr = 'y' + slider.value;

  mapSvg
    .selectAll('.countries')
    .attr('fill', (d) =>
      getCountryColor(d.properties.name),
    );
}
