// plot.js
import {
  plotSvg,
  plotWidth,
  height,
  margin,
  slider,
  display,
} from './globals.js';

let gdpDataMap;
let selectedCountry;

export function updateLinePlot() {
  const plotInnerWidth =
    plotWidth - margin.left - margin.right;
  const plotInnerHeight =
    height - margin.top - margin.bottom;

  const data = gdpDataMap.get(selectedCountry);

  if (!data) {
    plotSvg.selectAll('.line').remove();
    plotSvg
      .selectAll('.x-axis')
      .call(
        d3.axisBottom(
          d3.scaleLinear().range([0, plotInnerWidth]),
        ),
      );
    plotSvg
      .selectAll('.y-axis')
      .call(
        d3.axisLeft(
          d3.scaleLinear().range([plotInnerHeight, 0]),
        ),
      );
    return;
  }

  const years = Object.keys(data).map(
    (d) => +d.substring(1),
  );
  const values = Object.values(data);

  // Create scales
  const xScale = d3
    .scaleLinear()
    .domain(d3.extent(years))
    .range([0, plotInnerWidth]);
  const yScale = d3
    .scaleLinear()
    .domain(d3.extent(values))
    .range([plotInnerHeight, 0]);

  const startYear = Math.ceil(years[0] / 20) * 20;
  const endYear =
    Math.floor(years[years.length - 1] / 20) * 20;
  const tickValues = d3.range(startYear, endYear + 1, 20);

  // Update axes
  plotSvg
    .selectAll('.x-axis')
    .call(d3.axisBottom(xScale).tickValues(tickValues));
  plotSvg.selectAll('.y-axis').call(d3.axisLeft(yScale));

  // Create line generator
  const line = d3
    .line()
    .x((d) => xScale(d[0]))
    .y((d) => yScale(d[1]));

  // Prepare data pairs
  const plotData = years.map((year, i) => [
    year,
    values[i],
  ]);

  // Draw the line
  const update = plotSvg
    .selectAll('.line')
    .data([plotData]);

  update
    .join('path')
    .attr('class', 'line')
    .attr(
      'transform',
      `translate(${margin.left},${margin.top})`,
    )
    .attr('fill', 'none')
    .attr('stroke', 'blue')
    .attr('stroke-width', 1.5)
    .attr('d', line);

  // Draw the vertical line for the currently selected year
  const selectedYear = +slider.value;
  const selectedX = xScale(selectedYear);
  const selectedValue = data[`y${selectedYear}`];
  const selectedY = yScale(selectedValue);

  // Update or create the year-line
  const yearLine = plotSvg
    .selectAll('.year-line')
    .data([selectedX]);

  yearLine
    .join('line')
    .attr('class', 'year-line')
    .attr(
      'transform',
      `translate(${margin.left},${margin.top})`,
    )
    .attr('x1', (d) => d)
    .attr('x2', (d) => d)
    .attr('y1', 0)
    .attr('y2', plotInnerHeight)
    .attr('stroke', 'red')
    .attr('stroke-width', 1.5)
    .attr('stroke-dasharray', '4,4');

  // Draw the horizontal line
  const horizontalLine = plotSvg
    .selectAll('.horizontal-line')
    .data([selectedY]);

  horizontalLine
    .join('line')
    .attr('class', 'horizontal-line')
    .attr(
      'transform',
      `translate(${margin.left},${margin.top})`,
    )
    .attr('x1', 0)
    .attr('x2', selectedX)
    .attr('y1', (d) => d)
    .attr('y2', (d) => d)
    .attr('stroke', 'green')
    .attr('stroke-width', 1.5)
    .attr('stroke-dasharray', '4,4');

  // Display the value on the y-axis
  const valueText = plotSvg
    .selectAll('.value-text')
    .data([selectedY]);

  valueText
    .join('text')
    .attr('class', 'value-text')
    .attr(
      'transform',
      `translate(${margin.left},${margin.top})`,
    )
    .attr('x', -10) // Adjust as needed
    .attr('y', (d) => d)
    .attr('dy', '0.35em')
    .attr('text-anchor', 'end')
    .text(selectedValue)
    .style('fill', 'green')
    .style('font-size', '12px');

  // Update the plot title
  plotSvg
    .select('.plot-title')
    .text(`GDP Over Time: ${selectedCountry}`)
    .attr('x', plotInnerWidth / 2 + margin.left)
    .attr('y', margin.top / 2)
    .attr('text-anchor', 'middle')
    .style('font-size', '16px');
}

export function initPlot(_gdpDataMap, _selectedCountry) {
  gdpDataMap = _gdpDataMap;
  selectedCountry = _selectedCountry;

  // Create the line plot axis groups
  plotSvg
    .append('g')
    .attr('class', 'x-axis')
    .attr(
      'transform',
      `translate(${margin.left},${height - margin.bottom})`,
    );
  plotSvg
    .append('g')
    .attr('class', 'y-axis')
    .attr(
      'transform',
      `translate(${margin.left},${margin.top})`,
    );
  plotSvg.append('text').attr('class', 'plot-title');
}
